/**
 * plugin.js
 *
 * Copyright 2013 Web Power, www.webpower.nl
 * @author Arjan Haverkamp
 */

/* jshint unused:false */
/* global tinymce:true */


// changed for tinymce 6 2023 mirko boeer

tinymce.PluginManager.requireLangPack('codemirror')

tinymce.PluginManager.add('codemirror', function(editor, url) {

    function showSourceEditor() {
        editor.focus()
        editor.selection.collapse(true)

        let CaretMarker = "&#xFF;&#xFD;&#xFE;";

        let settings = {};
        settings.codemirror = editor.options.get("codemirror");
        if(typeof settings.codemirror !== "object")
          settings.codemirror = {};

        // Insert caret marker
        if (settings.codemirror.saveCursorPosition) {
            editor.selection.setContent('<span style="display: none;" class="CmCaReT">' + CaretMarker + '</span>')
        }

        var codemirrorWidth = (window.innerWidth * 80 / 100);
        if (settings.codemirror.width) {
            codemirrorWidth = settings.codemirror.width
        }

        var codemirrorHeight = (window.innerHeight * 70 / 100);
        if (settings.codemirror.height) {
            codemirrorHeight = settings.codemirror.height
        }

        var buttonsConfig =  [
                {
                    type: 'custom',
                    text: 'Ok',
                    name: 'codemirrorOk',
                    primary: true
                },
                {
                    type: 'cancel',
                    text: 'Cancel',
                    name: 'codemirrorCancel'
                }
            ]

        var config = {
            title: 'HTML source code',
            url: url + '/source.html',
            width: codemirrorWidth,
            height: codemirrorHeight,
            resizable: true,
            maximizable: true,
            saveCursorPosition: true,
            buttons: buttonsConfig
        }

        config.onAction = function (dialogApi, actionData) {
            if (actionData.name === 'codemirrorOk') {
                var doc = document.querySelectorAll('.tox-dialog__body-iframe iframe')[0]
                doc.contentWindow.submit()
                editor.undoManager.add()
                dialogApi.close();
            }
        }

        editor.windowManager.openUrl(config);
    }

    editor.ui.registry.addButton('codemirror', {
        icon: 'sourcecode',
        title: 'Source code',
        tooltip: 'Source code',
        onAction: showSourceEditor
    })

    editor.ui.registry.addMenuItem('codemirror', {
        icon: 'sourcecode',
        text: 'Source code',
        onAction: showSourceEditor
        //,context: 'tools'
    })

    editor.ui.registry.addContextMenu('codemirror', {
       update: (element) => ['BODY', 'P', 'DIV', 'TD', 'IMG'].contains(element.tagName) ? "codemirror" : ""
     });

  return {
    name: 'CodemirrorPlugin',
    url: 'https://www.superwebmailer.de/'
  };

})

